@@ -203,12 +203,6 @@ namespace Ogre
         Texture::loadImpl();
     }
 
-    /****************************************************************************************/
-    void D3D9Texture::freeInternalResources(void)
-    {
-        freeInternalResourcesImpl();
-    }
-
     /****************************************************************************************/
     void D3D9Texture::freeInternalResourcesImpl()
     {
@@ -314,12 +308,6 @@ namespace Ogre
         return instanceSize * mMapDeviceToTextureResources.size();
     }
 
-    /****************************************************************************************/
-    void D3D9Texture::createInternalResources(void)
-    {
-        createInternalResourcesImpl();
-    }
-
     /****************************************************************************************/
     void D3D9Texture::determinePool()
     {
